/// <reference types="node" />
/// <reference types="node" />
import type { PgClient, WithPgClient } from "@dataplan/pg";
import type { PromiseOrDirect } from "grafast";
import type { IncomingMessage } from "node:http";
import type { Socket } from "node:net";
import type { KeysOfType } from "./interfaces.js";
declare global {
    namespace Grafast {
        interface PgDatabaseAdaptorOptions {
        }
    }
    namespace GraphileConfig {
        /**
         * Details about the incoming GraphQL request - e.g. if it was sent over an
         * HTTP request, the request itself so headers can be interrogated.
         *
         * It's anticipated this will be expanded via declaration merging, e.g. if
         * your server is Koa then a `koaCtx` might be added.
         */
        interface GraphQLRequestContext {
            httpRequest?: IncomingMessage;
            socket?: Socket;
        }
        interface PgDatabaseConfiguration<TAdaptor extends keyof Grafast.PgDatabaseAdaptorOptions = keyof Grafast.PgDatabaseAdaptorOptions> {
            name: string;
            schemas?: string[];
            adaptor: TAdaptor;
            adaptorSettings?: Grafast.PgDatabaseAdaptorOptions[TAdaptor];
            /** The key on 'context' where the withPgClient function will be sourced */
            withPgClientKey: KeysOfType<GraphileBuild.GraphileResolverContext, WithPgClient>;
            listen?(topic: string): AsyncIterable<string>;
            /** Return settings to set in the session */
            pgSettings?: (graphqlRequestContext: GraphileConfig.GraphQLRequestContext) => {
                [key: string]: string;
            } | null;
            /** Settings to set in the session that performs introspection (during gather phase) */
            pgSettingsForIntrospection?: {
                [key: string]: string;
            } | null;
            /** The key on 'context' where the pgSettings for this DB will be sourced */
            pgSettingsKey?: KeysOfType<GraphileBuild.GraphileResolverContext, {
                [key: string]: string;
            } | null>;
        }
        interface Preset {
            pgSources?: ReadonlyArray<PgDatabaseConfiguration>;
        }
    }
}
/**
 * Get or build the 'withPgClient' callback function for a given database
 * source, caching it to make future lookups faster.
 */
export declare function getWithPgClientFromPgSource(source: GraphileConfig.PgDatabaseConfiguration): PromiseOrDirect<WithPgClient>;
export declare function withPgClientFromPgSource<T>(source: GraphileConfig.PgDatabaseConfiguration, pgSettings: {
    [key: string]: string;
} | null, callback: (client: PgClient) => T | Promise<T>): Promise<T>;
export declare function listenWithPgClientFromPgSource(source: GraphileConfig.PgDatabaseConfiguration, topic: string, callback: (event: any) => void): Promise<() => void>;
//# sourceMappingURL=pgSources.d.ts.map