"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgAllRowsPlugin = void 0;
require("graphile-build");
require("./PgTablesPlugin.js");
require("graphile-config");
const grafast_1 = require("grafast");
const graphile_export_1 = require("graphile-export");
const behavior_js_1 = require("../behavior.js");
const index_js_1 = require("../index.js");
const utils_js_1 = require("../utils.js");
exports.PgAllRowsPlugin = {
    name: "PgAllRowsPlugin",
    description: "Adds 'all rows' accessors for all table-like datasources.",
    version: index_js_1.version,
    // TODO: Requires PgTablesPlugin
    inflection: {
        add: {
            allRowsConnection(options, source) {
                return this.connectionField(this.camelCase(`all-${this.pluralize(this._singularizedSourceName(source))}`));
            },
            allRowsList(options, source) {
                return this.listField(this.camelCase(`all-${this.pluralize(this._singularizedSourceName(source))}`));
            },
        },
    },
    schema: {
        hooks: {
            GraphQLObjectType_fields(fields, build, context) {
                const { graphql: { GraphQLList, GraphQLNonNull }, } = build;
                const { fieldWithHooks } = context;
                if (!context.scope.isRootQuery) {
                    return fields;
                }
                for (const source of build.input.pgSources) {
                    if (source.parameters) {
                        // Skip functions
                        continue;
                    }
                    if (!source.find || source.isVirtual) {
                        continue;
                    }
                    const type = build.getTypeByName(build.inflection.tableType(source.codec));
                    if (!type) {
                        continue;
                    }
                    const behavior = (0, behavior_js_1.getBehavior)([
                        source.codec.extensions,
                        source.extensions,
                    ]);
                    const defaultBehavior = "connection -list";
                    if (build.behavior.matches(behavior, "query:list", defaultBehavior)) {
                        const fieldName = build.inflection.allRowsList(source);
                        fields = build.extend(fields, {
                            [fieldName]: fieldWithHooks({
                                fieldName,
                                fieldBehaviorScope: `query:list`,
                                isPgFieldSimpleCollection: true,
                                pgSource: source,
                            }, () => ({
                                type: new GraphQLList(new GraphQLNonNull(type)),
                                deprecationReason: (0, utils_js_1.tagToString)(source.extensions?.tags?.deprecated),
                                plan: (0, graphile_export_1.EXPORTABLE)((source) => function plan() {
                                    return source.find();
                                }, [source]),
                            })),
                        }, `Adding 'all rows' list field for PgSource ${source}`);
                    }
                    if (build.behavior.matches(behavior, "query:connection", defaultBehavior)) {
                        const fieldName = build.inflection.allRowsConnection(source);
                        const connectionType = build.getTypeByName(build.inflection.tableConnectionType(source.codec));
                        if (connectionType) {
                            fields = build.extend(fields, {
                                [fieldName]: fieldWithHooks({
                                    fieldName,
                                    fieldBehaviorScope: `query:connection`,
                                    isPgFieldConnection: true,
                                    pgSource: source,
                                }, () => ({
                                    type: connectionType,
                                    description: `Reads and enables pagination through a set of \`${build.inflection.tableType(source.codec)}\`.`,
                                    deprecationReason: (0, utils_js_1.tagToString)(source.extensions?.tags?.deprecated),
                                    plan: (0, graphile_export_1.EXPORTABLE)((connection, source) => function plan() {
                                        return connection(source.find());
                                    }, [grafast_1.connection, source]),
                                })),
                            }, `Adding 'all rows' connection field for PgSource ${source}`);
                        }
                    }
                }
                return fields;
            },
        },
    },
};
//# sourceMappingURL=PgAllRowsPlugin.js.map