import "graphile-build";
import type { PgEnumTypeCodec, PgRecordTypeCodecSpec, PgTypeCodec, PgTypeColumn } from "@dataplan/pg";
import type { PluginHook } from "graphile-config";
import type { PgAttribute, PgClass, PgType } from "pg-introspection";
declare global {
    namespace GraphileBuild {
        interface Inflection {
            classCodecName(details: {
                pgClass: PgClass;
                databaseName: string;
            }): string;
            typeCodecName(details: {
                pgType: PgType;
                databaseName: string;
            }): string;
            scalarCodecTypeName(this: Inflection, codec: PgTypeCodec<undefined, any, any, undefined>): string;
            enumType(this: Inflection, codec: PgEnumTypeCodec<any>): string;
            enumValue(this: Inflection, value: string, codec: PgEnumTypeCodec<any>): string;
            rangeBoundType(input: {
                codec: PgTypeCodec<undefined, any, any, undefined>;
                underlyingTypeName: string;
            }): string;
            rangeType(input: {
                codec: PgTypeCodec<undefined, any, any, undefined>;
                underlyingTypeName: string;
            }): string;
        }
        interface BuildInput {
            /**
             * A non-exhaustive list of codecs, please walk pgSources for more.
             * Typically useful for the codecs that aren't linked to a source (e.g.
             * those defining an union or interface)
             */
            pgCodecs?: PgTypeCodec<any, any, any, any>[];
        }
        interface Build {
            allPgCodecs: Set<PgTypeCodec<any, any, any, any>>;
        }
        interface ScopeObject {
            isPgRangeType?: boolean;
            isPgRangeBoundType?: boolean;
        }
        interface ScopeInputObject {
            isPgRangeInputType?: boolean;
            isPgRangeBoundInputType?: boolean;
            pgCodec?: PgTypeCodec<any, any, any, any>;
        }
    }
    namespace GraphileConfig {
        interface GatherHelpers {
            pgCodecs: {
                getCodecFromClass(databaseName: string, pgClassId: string): Promise<PgTypeCodec<any, any, any> | null>;
                getCodecFromType(databaseName: string, pgTypeId: string, pgTypeModifier?: string | number | null): Promise<PgTypeCodec<any, any, any, any> | null>;
            };
        }
        interface GatherHooks {
            pgCodecs_PgTypeCodec: PluginHook<(event: {
                databaseName: string;
                pgCodec: PgTypeCodec<any, any, any, any>;
                pgClass?: PgClass;
                pgType: PgType;
            }) => Promise<void> | void>;
            pgCodecs_column: PluginHook<(event: {
                databaseName: string;
                pgClass: PgClass;
                pgAttribute: PgAttribute;
                column: PgTypeColumn<any>;
            }) => Promise<void> | void>;
            pgCodecs_recordType_spec: PluginHook<(event: {
                databaseName: string;
                pgClass: PgClass;
                spec: PgRecordTypeCodecSpec<any>;
            }) => Promise<void> | void>;
            pgCodecs_enumType_extensions: PluginHook<(event: {
                databaseName: string;
                pgType: PgType;
                extensions: any;
            }) => Promise<void> | void>;
            pgCodecs_rangeOfCodec_extensions: PluginHook<(event: {
                databaseName: string;
                pgType: PgType;
                innerCodec: PgTypeCodec<any, any, any, any>;
                extensions: any;
            }) => Promise<void> | void>;
            pgCodecs_domainOfCodec_extensions: PluginHook<(event: {
                databaseName: string;
                pgType: PgType;
                innerCodec: PgTypeCodec<any, any, any, any>;
                extensions: any;
            }) => Promise<void> | void>;
            pgCodecs_listOfCodec_extensions: PluginHook<(event: {
                databaseName: string;
                pgType: PgType;
                innerCodec: PgTypeCodec<any, any, any, any>;
                extensions: any;
            }) => Promise<void> | void>;
        }
    }
}
declare module "@dataplan/pg" {
    interface PgSourceRelationExtensions {
        originalName?: string;
    }
}
export declare const PgCodecsPlugin: GraphileConfig.Plugin;
//# sourceMappingURL=PgCodecsPlugin.d.ts.map