"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgConditionCustomFieldsPlugin = void 0;
require("graphile-config");
const graphile_export_1 = require("graphile-export");
const behavior_js_1 = require("../behavior.js");
const index_js_1 = require("../index.js");
exports.PgConditionCustomFieldsPlugin = {
    name: "PgConditionCustomFieldsPlugin",
    description: "Add GraphQL conditions based on 'filterable' PostgreSQL functions",
    version: index_js_1.version,
    schema: {
        hooks: {
            GraphQLInputObjectType_fields(fields, build, context) {
                const { inflection, sql } = build;
                const { scope: { isPgCondition, pgCodec }, fieldWithHooks, } = context;
                if (!isPgCondition ||
                    !pgCodec ||
                    !pgCodec.columns ||
                    pgCodec.isAnonymous) {
                    return fields;
                }
                const functionSources = build.input.pgSources.filter((source) => {
                    if (source.codec.columns)
                        return false;
                    if (source.codec.arrayOfCodec)
                        return false;
                    if (source.codec.rangeOfCodec)
                        return false;
                    const parameters = source.parameters;
                    if (!parameters || parameters.length < 1)
                        return false;
                    if (parameters.some((p, i) => i > 0 && p.required))
                        return false;
                    if (parameters[0].codec !== pgCodec)
                        return false;
                    if (!source.isUnique)
                        return false;
                    const behavior = (0, behavior_js_1.getBehavior)([source.extensions]);
                    return build.behavior.matches(behavior, "proc:filterBy", "-proc:filterBy");
                });
                return build.extend(fields, functionSources.reduce((memo, pgFieldSource) => {
                    const fieldName = inflection.computedColumnField({
                        source: pgFieldSource,
                    });
                    const type = build.getGraphQLTypeByPgCodec(pgFieldSource.codec, "input");
                    if (!type)
                        return memo;
                    memo = build.extend(memo, {
                        [fieldName]: fieldWithHooks({
                            fieldName,
                            fieldBehaviorScope: "proc:filterBy",
                            isPgConnectionConditionInputField: true,
                            pgFieldSource,
                        }, {
                            description: build.wrapDescription(`Checks for equality with the object’s \`${fieldName}\` field.`, "field"),
                            type,
                            applyPlan: (0, graphile_export_1.EXPORTABLE)((pgFieldSource, sql) => function plan($condition, val) {
                                if (typeof pgFieldSource.source !== "function") {
                                    throw new Error("Invalid computed column source");
                                }
                                const expression = sql `${pgFieldSource.source({
                                    placeholder: $condition.alias,
                                })}`;
                                if (val.getRaw().evalIs(null)) {
                                    $condition.where(sql `${expression} is null`);
                                }
                                else {
                                    $condition.where(sql `${expression} = ${$condition.placeholder(val.get(), pgFieldSource.codec)}`);
                                }
                            }, [pgFieldSource, sql]),
                        }),
                    }, `Adding computed column condition argument for ${pgCodec.name}`);
                    return memo;
                }, {}), `Adding computed column filterable functions to condition for '${pgCodec.name}'`);
                return fields;
            },
        },
    },
};
//# sourceMappingURL=PgConditionCustomFieldsPlugin.js.map