"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyOrderToPlan = exports.PgConnectionArgOrderByPlugin = void 0;
require("./PgTablesPlugin.js");
require("graphile-config");
const grafast_1 = require("grafast");
const graphile_export_1 = require("graphile-export");
const util_1 = require("util");
const behavior_js_1 = require("../behavior.js");
const index_js_1 = require("../index.js");
// TODO: rename this, it's not just for connections
exports.PgConnectionArgOrderByPlugin = {
    name: "PgConnectionArgOrderByPlugin",
    description: "Adds the 'orderBy' argument to connections and simple collections",
    version: index_js_1.version,
    inflection: {
        add: {
            orderByType(options, typeName) {
                return this.upperCamelCase(`${typeName}-order-by`);
            },
        },
    },
    schema: {
        hooks: {
            init(_, build) {
                const { inflection, pgCodecMetaLookup } = build;
                pgCodecMetaLookup.forEach((meta, codec) => {
                    if (!codec.columns || codec.isAnonymous)
                        return;
                    const behavior = (0, behavior_js_1.getBehavior)(codec.extensions);
                    // TODO: should this be `type:order` or similar?
                    if (!build.behavior.matches(behavior, "order", "order")) {
                        return;
                    }
                    const tableTypeName = inflection.tableType(codec);
                    /* const TableOrderByType = */
                    const typeName = inflection.orderByType(tableTypeName);
                    build.registerEnumType(typeName, {
                        pgCodec: codec,
                        isPgRowSortEnum: true,
                    }, () => ({
                        description: build.wrapDescription(`Methods to use when ordering \`${tableTypeName}\`.`, "type"),
                        values: {
                            [inflection.builtin("NATURAL")]: {
                                extensions: {
                                    graphile: {
                                        // NATURAL means to not change the sort order
                                        applyPlan: (0, graphile_export_1.EXPORTABLE)(() => () => { }, []),
                                    },
                                },
                            },
                        },
                    }), `Adding connection "orderBy" argument for ${codec.name}.`);
                });
                return _;
            },
            GraphQLObjectType_fields_field_args(args, build, context) {
                const { extend, getTypeByName, graphql: { GraphQLList, GraphQLNonNull }, inflection, } = build;
                const { scope, Self } = context;
                const { fieldName, isPgFieldConnection, isPgFieldSimpleCollection, pgSource, pgFieldCodec, } = scope;
                if (!isPgFieldConnection && !isPgFieldSimpleCollection) {
                    return args;
                }
                const codec = pgFieldCodec ?? pgSource?.codec;
                const isSuitableSource = pgSource && pgSource.codec.columns && !pgSource.isUnique;
                const isSuitableCodec = codec &&
                    (isSuitableSource ||
                        (!pgSource && codec?.polymorphism?.mode === "union")) &&
                    codec.columns;
                if (!isSuitableCodec) {
                    return args;
                }
                const behavior = (0, behavior_js_1.getBehavior)([
                    scope,
                    codec?.extensions,
                    pgSource?.extensions,
                ]);
                if (!build.behavior.matches(behavior, "order", pgSource?.parameters ? "" : "order")) {
                    return args;
                }
                const tableTypeName = inflection.tableType(codec);
                const tableOrderByTypeName = inflection.orderByType(tableTypeName);
                const TableOrderByType = getTypeByName(tableOrderByTypeName);
                if (!TableOrderByType) {
                    return args;
                }
                return extend(args, {
                    orderBy: {
                        description: build.wrapDescription(`The method to use when ordering \`${tableTypeName}\`.`, "arg"),
                        type: new GraphQLList(new GraphQLNonNull(TableOrderByType)),
                        applyPlan: isPgFieldConnection
                            ? (0, graphile_export_1.EXPORTABLE)((applyOrderToPlan, tableOrderByTypeName) => function plan(_, $connection, val, info) {
                                const $value = val.getRaw();
                                const $select = $connection.getSubplan();
                                applyOrderToPlan($select, $value, info.schema.getType(tableOrderByTypeName));
                                return null;
                            }, [exports.applyOrderToPlan, tableOrderByTypeName])
                            : (0, graphile_export_1.EXPORTABLE)((applyOrderToPlan, tableOrderByTypeName) => function plan(_, $select, val, info) {
                                const $value = val.getRaw();
                                applyOrderToPlan($select, $value, info.schema.getType(tableOrderByTypeName));
                                return null;
                            }, [exports.applyOrderToPlan, tableOrderByTypeName]),
                    },
                }, `Adding 'orderBy' argument to field '${fieldName}' of '${Self.name}'`);
            },
        },
    },
};
exports.applyOrderToPlan = (0, graphile_export_1.EXPORTABLE)((getEnumValueConfig, inspect) => ($select, $value, TableOrderByType) => {
    const val = $value.eval();
    if (!Array.isArray(val)) {
        throw new Error("Invalid!");
    }
    val.forEach((order) => {
        const config = getEnumValueConfig(TableOrderByType, order);
        const plan = config?.extensions?.graphile?.applyPlan;
        if (typeof plan !== "function") {
            console.error(`Internal server error: invalid orderBy configuration: expected function, but received ${inspect(plan)}`);
            throw new Error("Internal server error: invalid orderBy configuration");
        }
        plan($select);
    });
}, [grafast_1.getEnumValueConfig, util_1.inspect]);
//# sourceMappingURL=PgConnectionArgOrderByPlugin.js.map