import "graphile-build";
import { PgExecutor } from "@dataplan/pg";
import type { PromiseOrDirect } from "grafast";
import type { PluginHook } from "graphile-config";
import type { Introspection, PgAttribute, PgAuthMembers, PgClass, PgConstraint, PgDepend, PgDescription, PgEnum, PgExtension, PgIndex, PgInherits, PgLanguage, PgNamespace, PgProc, PgRange, PgRoles, PgType } from "pg-introspection";
export declare type PgEntityWithId = PgNamespace | PgClass | PgConstraint | PgProc | PgRoles | PgType | PgEnum | PgExtension | PgExtension | PgIndex | PgLanguage;
declare global {
    namespace GraphileBuild {
        interface GraphileBuildGatherOptions {
            /**
             * Should we attempt to install the watch fixtures into the database?
             *
             * Default: true
             */
            installWatchFixtures?: boolean;
        }
    }
    namespace GraphileConfig {
        interface GatherHelpers {
            pgIntrospection: {
                getIntrospection(): Promise<Array<{
                    introspection: Introspection;
                    database: GraphileConfig.PgDatabaseConfiguration;
                }>>;
                getDatabase(databaseName: string): Promise<{
                    introspection: Introspection;
                    database: GraphileConfig.PgDatabaseConfiguration;
                }>;
                getExecutorForDatabase(databaseName: string): PgExecutor;
                getNamespace(databaseName: string, id: string): Promise<PgNamespace | undefined>;
                getClasses(databaseName: string): Promise<PgClass[]>;
                getClass(databaseName: string, id: string): Promise<PgClass | undefined>;
                getConstraint(databaseName: string, id: string): Promise<PgConstraint | undefined>;
                getProc(databaseName: string, id: string): Promise<PgProc | undefined>;
                getRoles(databaseName: string, id: string): Promise<PgRoles | undefined>;
                getType(databaseName: string, id: string): Promise<PgType | undefined>;
                getEnum(databaseName: string, id: string): Promise<PgEnum | undefined>;
                getExtension(databaseName: string, id: string): Promise<PgExtension | undefined>;
                getIndex(databaseName: string, id: string): Promise<PgIndex | undefined>;
                getLanguage(databaseName: string, id: string): Promise<PgLanguage | undefined>;
                getAttribute(databaseName: string, classId: string, attributeNumber: number): Promise<PgAttribute | undefined>;
                getAttributesForClass(databaseName: string, classId: string): Promise<PgAttribute[]>;
                getConstraintsForClass(databaseName: string, classId: string): Promise<PgConstraint[]>;
                getForeignConstraintsForClass(databaseName: string, classId: string): Promise<PgConstraint[]>;
                getInheritedForClass(databaseName: string, classId: string): Promise<PgInherits[]>;
                getNamespaceByName(databaseName: string, namespaceName: string): Promise<PgNamespace | undefined>;
                getClassByName(databaseName: string, namespaceName: string, tableName: string): Promise<PgClass | undefined>;
                getTypeByArray(databaseName: string, arrayId: string): Promise<PgType | undefined>;
                getEnumsForType(databaseName: string, typeId: string): Promise<PgEnum[]>;
                getRangeByType(databaseName: string, typeId: string): Promise<PgRange | null>;
            };
        }
        interface GatherHooks {
            pgIntrospection_introspection: PluginHook<(event: {
                introspection: Introspection;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_namespace: PluginHook<(event: {
                entity: PgNamespace;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_class: PluginHook<(event: {
                entity: PgClass;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_attribute: PluginHook<(event: {
                entity: PgAttribute;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_constraint: PluginHook<(event: {
                entity: PgConstraint;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_proc: PluginHook<(event: {
                entity: PgProc;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_role: PluginHook<(event: {
                entity: PgRoles;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_auth_member: PluginHook<(event: {
                entity: PgAuthMembers;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_type: PluginHook<(event: {
                entity: PgType;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_enum: PluginHook<(event: {
                entity: PgEnum;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_extension: PluginHook<(event: {
                entity: PgExtension;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_index: PluginHook<(event: {
                entity: PgIndex;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_language: PluginHook<(event: {
                entity: PgLanguage;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_range: PluginHook<(event: {
                entity: PgRange;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_depend: PluginHook<(event: {
                entity: PgDepend;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
            pgIntrospection_description: PluginHook<(event: {
                entity: PgDescription;
                databaseName: string;
            }) => PromiseOrDirect<void>>;
        }
    }
}
export declare const PgIntrospectionPlugin: GraphileConfig.Plugin;
//# sourceMappingURL=PgIntrospectionPlugin.d.ts.map