"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgOrderByPrimaryKeyPlugin = void 0;
require("./PgTablesPlugin.js");
require("graphile-config");
const graphile_export_1 = require("graphile-export");
const index_js_1 = require("../index.js");
exports.PgOrderByPrimaryKeyPlugin = {
    name: "PgOrderByPrimaryKeyPlugin",
    description: "Adds ordering by the table's primary key",
    version: index_js_1.version,
    schema: {
        hooks: {
            GraphQLEnumType_values(values, build, context) {
                const { extend, inflection, sql, options } = build;
                const { scope: { isPgRowSortEnum, pgCodec }, } = context;
                const { orderByNullsLast } = options;
                if (!isPgRowSortEnum ||
                    !pgCodec ||
                    !pgCodec.columns ||
                    pgCodec.isAnonymous) {
                    return values;
                }
                const sources = build.input.pgSources.filter((s) => s.codec === pgCodec && !s.parameters);
                if (sources.length < 1) {
                    return values;
                }
                const primaryKey = sources[0].uniques.find((source) => source.isPrimary);
                if (!primaryKey) {
                    return values;
                }
                const primaryKeyColumns = primaryKey.columns;
                return extend(values, {
                    [inflection.builtin("PRIMARY_KEY_ASC")]: {
                        extensions: {
                            graphile: {
                                applyPlan: (0, graphile_export_1.EXPORTABLE)((orderByNullsLast, pgCodec, primaryKeyColumns, sql) => (step) => {
                                    primaryKeyColumns.forEach((columnName) => {
                                        const column = pgCodec.columns[columnName];
                                        step.orderBy({
                                            codec: column.codec,
                                            fragment: sql `${step.alias}.${sql.identifier(columnName)}`,
                                            direction: "ASC",
                                            ...(orderByNullsLast != null
                                                ? {
                                                    nulls: orderByNullsLast ? "LAST" : "FIRST",
                                                }
                                                : null),
                                        });
                                    });
                                    step.setOrderIsUnique();
                                }, [orderByNullsLast, pgCodec, primaryKeyColumns, sql]),
                            },
                        },
                    },
                    [inflection.builtin("PRIMARY_KEY_DESC")]: {
                        extensions: {
                            graphile: {
                                applyPlan: (0, graphile_export_1.EXPORTABLE)((orderByNullsLast, pgCodec, primaryKeyColumns, sql) => (step) => {
                                    primaryKeyColumns.forEach((columnName) => {
                                        const column = pgCodec.columns[columnName];
                                        step.orderBy({
                                            codec: column.codec,
                                            fragment: sql `${step.alias}.${sql.identifier(columnName)}`,
                                            direction: "DESC",
                                            ...(orderByNullsLast != null
                                                ? {
                                                    nulls: orderByNullsLast ? "LAST" : "FIRST",
                                                }
                                                : null),
                                        });
                                    });
                                    step.setOrderIsUnique();
                                }, [orderByNullsLast, pgCodec, primaryKeyColumns, sql]),
                            },
                        },
                    },
                }, `Adding primary key asc/desc sort to table '${pgCodec.name}'`);
            },
        },
    },
};
//# sourceMappingURL=PgOrderByPrimaryKeyPlugin.js.map