"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgSmartCommentsPlugin = void 0;
require("graphile-config");
require("./PgCodecsPlugin.js");
require("./PgProceduresPlugin.js");
require("./PgRelationsPlugin.js");
require("./PgTablesPlugin.js");
const index_js_1 = require("../index.js");
const mergeTags_js_1 = require("../mergeTags.js");
exports.PgSmartCommentsPlugin = {
    name: "PgSmartCommentsPlugin",
    description: "Applies smart comments and descriptions to various resources",
    version: index_js_1.version,
    provides: ["smart-tags"],
    gather: {
        namespace: "pgSmartComments",
        helpers: {},
        hooks: {
            pgTables_unique(info, event) {
                const { pgConstraint, unique } = event;
                applyTags(pgConstraint, unique);
            },
            pgTables_PgSourceBuilder_options(info, event) {
                const { pgClass, options } = event;
                applyTags(pgClass, options);
            },
            pgRelations_relation(info, event) {
                const { pgConstraint, relation } = event;
                const { tags } = pgConstraint.getTagsAndDescription();
                relation.extensions = relation.extensions || {
                    tags: Object.create(null),
                };
                relation.extensions.tags =
                    relation.extensions.tags || Object.create(null);
                // Clone the tags because we use the same tags on both relations
                // (in both directions) but don't want modifications made to one
                // to affect the other.
                (0, mergeTags_js_1.mergeTags)(relation.extensions.tags, JSON.parse(JSON.stringify(tags)));
            },
            pgCodecs_column(info, event) {
                const { pgAttribute, column } = event;
                applyTags(pgAttribute, column);
            },
            pgCodecs_recordType_spec(info, event) {
                const { pgClass, spec } = event;
                const extensions = spec.extensions ?? Object.create(null);
                if (!spec.extensions) {
                    spec.extensions = extensions;
                }
                const typeTagsAndDescription = pgClass
                    .getType()
                    .getTagsAndDescription();
                const classTagsAndDescription = pgClass.getTagsAndDescription();
                if (!extensions.description) {
                    extensions.description =
                        typeTagsAndDescription.description ||
                            classTagsAndDescription.description;
                }
                extensions.tags = extensions.tags || Object.create(null);
                (0, mergeTags_js_1.mergeTags)(extensions.tags, typeTagsAndDescription.tags);
                (0, mergeTags_js_1.mergeTags)(extensions.tags, classTagsAndDescription.tags);
            },
            pgCodecs_rangeOfCodec_extensions(info, event) {
                const { pgType, extensions } = event;
                applyTagsToExtensions(pgType, extensions);
            },
            pgCodecs_domainOfCodec_extensions(info, event) {
                const { pgType, extensions } = event;
                applyTagsToExtensions(pgType, extensions);
            },
            pgCodecs_listOfCodec_extensions(info, event) {
                const { pgType, extensions } = event;
                applyTagsToExtensions(pgType, extensions);
            },
            pgProcedures_functionSource_options(info, event) {
                const { pgProc, options } = event;
                applyTags(pgProc, options);
            },
            pgProcedures_PgSource_options(info, event) {
                const { pgProc, options } = event;
                applyTags(pgProc, options);
            },
        },
    },
};
function applyTags(entity, config) {
    config.extensions = config.extensions || { tags: Object.create(null) };
    applyTagsToExtensions(entity, config.extensions);
}
function applyTagsToExtensions(entity, extensions) {
    const { tags, description } = entity.getTagsAndDescription();
    extensions.tags = extensions.tags || Object.create(null);
    if (!extensions.description) {
        extensions.description = description;
    }
    (0, mergeTags_js_1.mergeTags)(extensions.tags, tags);
}
//# sourceMappingURL=PgSmartCommentsPlugin.js.map